import Base from '../annotation/base';
import { getActualPoint, getClickPoint, createSvg, createElement } from '../annotation/utils';
import { onClickOutside } from '../ui_utils'

export default class ListBox extends Base {
  
  constructor ({
    container,
    annotation,
    page,
    viewport,
    scale,
    eventBus,
    layer,
    show = false
  }) {
    super({
      container,
      annotation,
      page,
      viewport,
      scale,
      eventBus,
      show
    })

    this.layer = layer
    this.hidden = true
    this.initial = false
    this.outline = null

    this.start = null
    this.end = null

    this.newStart = null
    this.newEnd = null

    this.startCircle = null
    this.endCircle = null

    this.show = show
    this.first = true

    this.onMousedown = this.handleMouseDown.bind(this)
    this.onMouseup = this.handleMouseUp.bind(this)
    this.onMousemove = this.handleMouseMove.bind(this)
    this.onDelete = this.handleDelete.bind(this)
    this.render()
  }

  render () {
    this.eventBus._on('listBoxPropertyPanelChanged', this.handlePropertyPanel.bind(this))
    this.eventBus._on('setProperty', this.setProperty.bind(this))
    this.eventBus._on('changeSelectAvailable', this.changeSelectAvailable.bind(this))

    for (let key in this.annotation) {
      if (key === 'background-color') {
        let newName = 'backgroundColor'
        this.annotation[newName] = this.annotation[key]
        delete this.annotation[key]
      }
      if (key === 'fieldname') {
        let newKey = 'fieldName'
        this.annotation[newKey] = this.annotation[key]
        delete this.annotation[key]
      }
      if (key === 'opt') {
        let newKey = 'options'
        let options = []
        if (Array.isArray(this.annotation[key])) {
          for (const item of this.annotation[key]) {
            options.push(item.value ?? item.key)
          }
        } else {
          options.push(this.annotation[key].value)
        }
        this.annotation[newKey] = options
        delete this.annotation[key]
      }
      if (!this.annotation['opt'] && !this.annotation['options']) {
        this.annotation.options = []
      }
      if (key === 'options' && typeof this.annotation.options === 'string') {
        this.annotation.options = this.annotation.options.split(',')
      }
      if (key === 'type' && this.annotation[key] === 'listbox') {
        this.annotation[key] = 'list-box'
      }
      if (key === 'flags') {
        let newKey = 'isHidden'
        this.annotation[newKey] = this.annotation[key].indexOf('noview') > -1 || this.annotation[key].indexOf('hidden') > -1 ? '1' : '0'
        delete this.annotation[key]
      }
      if (key === 'select') {
        this.annotation[key] = (parseInt(this.annotation[key]) + 1).toString()
      }
      if (key === 'textStyle') {
        const textStyle = this.annotation[key]
        if (textStyle.fontColor) this.annotation.textColor = textStyle.fontColor
        if (textStyle.fontSize) this.annotation.fontSize = parseInt(textStyle.fontSize).toString()
        if (textStyle.fontName && textStyle.fontName.length > 0) {
          const fontArr = [ 'Courier', 'Courier-Bold', 'Courier-BoldOblique', 'Courier-Oblique', 'Helvetica', 'Helvetica-Bold', 'Helvetica-BoldOblique', 'Helvetica-Oblique', 'Times-Roman', 'Times-Bold', 'Times-BoldItalic', 'Times-Italic' ]
          this.annotation.fontName = 'Helvetica'
          for (const item of fontArr) {
            if (item.toLowerCase() === textStyle.fontName.toLowerCase()) {
              this.annotation.fontName = item
            }
          }
        } else {
          this.annotation.fontName = 'Helvetica'
        }
      }
    }
    if (!this.annotation['select']) {
      this.annotation.select = ''
    }
    const annotation = this.annotation
    console.log(annotation)

    const { start, end } = this.getActualRect(
      this.viewport,
      this.scale
    )
    this.start = start
    this.end = end

    const rect = this.calculate(start, end)
    const actualbdwidth = (this.annotation.lineWidth || this.annotation.width || 2) * this.scale
    
    const annotationContainer = document.createElement('div')
    annotationContainer.id = annotation.name
    annotationContainer.className = 'annotation'
    annotationContainer.style.top = rect.top + 'px'
    annotationContainer.style.left = rect.left + 'px'
    annotationContainer.style.width = rect.width + 'px'
    annotationContainer.style.height = rect.height + 'px'
    this.annotationContainer = annotationContainer
    this.annotationContainer.addEventListener('mousedown', this.handleClick.bind(this))

    let borderElement = createElement(
      'div',
      {
        left: 0,
        top: 0,
        width: `${Math.abs(rect.width)}px`,
        height: `${Math.abs(rect.height)}px`,
        backgroundColor: annotation.backgroundColor,
        position: 'relative'
      }
    )

    this.borderElement = borderElement

    let selectElement = createElement(
      'div',
      {
        width: `${Math.abs(rect.width)}px`,
        height: `${Math.abs(rect.height)}px`,
        borderColor: '#43474D',
        borderWidth: '1px',
        borderStyle: 'solid',
        borderRadius: '0',
        cursor: 'pointer',
        background: 'transparent',
        fontSize: annotation.fontSize + 'px',
        fontFamily: annotation.fontName,
        color: annotation.textColor || '#001A4E',
        overflow: 'auto'
      }
    )
    this.selectElement = selectElement

    this.borderElement.append(this.selectElement)
    this.annotationContainer.append(this.borderElement)
    this.container.append(this.annotationContainer)

    // 添加选项
    if (annotation.options && annotation.options.length > 0) {
      for (var i = 0; i < annotation.options.length; i++) {
        const optionElement = createElement(
          'div',
          {
            width: '100%',
            height: '30px',
            lineHeight: '30px',
            borderBottom: '1px solid #666666',
            overflow: 'hidden',
            textOverflow: 'ellipsis',
            whiteSpace: 'nowrap'
          },
          {
            class: 'option'
          }
        )
        optionElement.innerText = annotation.options[i]
        if (this.annotation.select - 1 === i) {
          optionElement.style.backgroundColor = '#dcdcdc'
        }
        this.selectElement.appendChild(optionElement)
      }
    }

    this.outerLineContainer = document.createElement('div')
    this.outerLineContainer.className = 'outline-container'
    this.deletetButton = createSvg(
      "svg",
      {
        width: "30",
        height: "30",
        viewBox: "0 0 30 30",
        fill: "none",
        class: 'delete-button'
      },
      {
        width: '30px',
        height: '30px'
      }
    );
    this.deletetButton.addEventListener('click', this.onDelete)
    this.deletetButton.innerHTML = this.deleteSvgStr

    const left = (rect.left + rect.width - 30) + 'px'
    const top = (rect.top + rect.height + 8) + 'px'
    this.deletetButton.style.left = left
    this.deletetButton.style.top = top
    
    const outerLine = document.createElementNS("http://www.w3.org/2000/svg", "svg");

    outerLine.style.position = 'absolute'
    outerLine.style.zIndex = 1
    outerLine.style.left = `${rect.left - 8}px`
    outerLine.style.top = `${rect.top - 8}px`
    outerLine.style.width = `${rect.width + 8 * 2}px`
    outerLine.style.height = `${rect.height + 8 * 2}px`
    this.outerLine = outerLine
    
    this.moveRect = createSvg(
      "rect",
      {
        class: "move",
        'data-id': "move",
        stroke: "#4982E6",
        'stroke-width': 1,
        'fill-opacity': 0,
        width: rect.width + 8,
        height: rect.height + 8,
        x: 4,
        y: 4
      }
    );
    
    this.topLeftRect = createSvg(
      "rect",
      {
        class: "nw-resize",
        'data-id': "nw-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: 1,
        y: 1
      }
    );

    this.LeftRect = createSvg(
      "rect",
      {
        class: "w-resize",
        'data-id': "w-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: 1,
        y: rect.height / 2 + 5
      }
    );

    this.bottomLeftRect = createSvg(
      "rect",
      {
        class: "sw-resize",
        'data-id': "sw-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: 1,
        y: rect.height + 9
      }
    );

    this.bottomRect = createSvg(
      "rect",
      {
        class: "s-resize",
        'data-id': "s-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width / 2 + 5,
        y: rect.height + 9
      }
    );
    
    this.bottomRightRect = createSvg(
      "rect",
      {
        class: "se-resize",
        'data-id': "se-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width + 9,
        y: rect.height + 9
      }
    );

    this.rightRect = createSvg(
      "rect",
      {
        class: "e-resize",
        'data-id': "e-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width + 9,
        y: rect.height / 2 + 5
      }
    );

    this.topRightRect = createSvg(
      "rect",
      {
        class: "ne-resize",
        'data-id': "ne-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width + 9,
        y: 1
      }
    );

    this.topRect = createSvg(
      "rect",
      {
        class: "n-resize",
        'data-id': "n-resize",
        fill: "#FFFFFF",
        stroke: "#4982E6",
        'stroke-width': 1,
        width: 6,
        height: 6,
        x: rect.width / 2 + 4,
        y: 1
      }
    );

    this.outerLine.append(this.moveRect)
    this.outerLine.append(this.topLeftRect)
    this.outerLine.append(this.LeftRect)
    this.outerLine.append(this.bottomLeftRect)
    this.outerLine.append(this.bottomRect)
    this.outerLine.append(this.bottomRightRect)
    this.outerLine.append(this.rightRect)
    this.outerLine.append(this.topRightRect)
    this.outerLine.append(this.topRect)
    this.outerLineContainer.append(this.outerLine)
    this.outerLineContainer.append(this.deletetButton)
    this.initial = true

    if (this.layer.toolMode !== 'form') {
      if (this.annotation['isHidden'] === '1') {
        this.annotationContainer.style.display = 'none'
        this.annotationContainer.style.pointerEvents = 'none'
      } else if (this.annotation['isHidden'] === '0') {
        this.annotationContainer.style.display = 'block'
        this.annotationContainer.style.pointerEvents = 'auto'
      }
    }
    this.eventBus.dispatch('propertyChange', this.annotation)

    if (this.show) this.handleClick()
  }

  getActualRect (viewport, s,) {
    const annotation = this.annotation
    const [x1, y1, x2, y2] = annotation.rect.split(',')
    
    const start = getActualPoint(
      {
        x: x1,
        y: y1
      },
      viewport,
      s
    )
    const end = getActualPoint(
      {
        x: x2,
        y: y2
      },
      viewport,
      s
    )
    return {
      start,
      end
    }
  }

  rectCalc ({ start, end }) {
    return {
      top: start.y < end.y ? start.y : end.y,
      left: start.x < end.x ? start.x : end.x,
      width: Math.abs(end.x - start.x),
      height: Math.abs(end.y - start.y),
    };
  };

  calculate (start, end) {
    const initRect = this.rectCalc({ start, end });
    const actualbdwidth = (this.annotation.width || this.annotation.lineWidth || 2) * this.scale

    return {
      top: initRect.top - actualbdwidth,
      left: initRect.left - actualbdwidth,
      width: initRect.width + actualbdwidth * 2,
      height: initRect.height + actualbdwidth * 2,
    }
  }

  getInitialPoint () {
    const s = this.scale
    const startX = this.start.x
    const startY = this.start.y
    const endX = this.end.x
    const endY = this.end.y

    const { width, height, rotation  } = this.viewport;
    let x1, y1, x2, y2;
    if (rotation === 0) {
      x1 = startX / s;
      y1 = (height - startY) / s; 
      x2 = endX / s;
      y2 = (height - endY) / s;
    } else if (rotation === 90) {
      x1 = startY / s;
      y1 = startX / s;
      x2 = endY / s;
      y2 = endX / s;
    } else if (rotation === 180) {
      x1 =( width - startX) / s;
      y1 = startY / s;
      x2 = (width - endX) / s;
      y2 = endY / s;
    } else {
      x1 = (height - startY) / s
      y1 = (width - startX) / s
      x2 = (height - endY) / s
      y2 = (width - endX)  / s
    }
    return {
      start: {
        x: x1,
        y: y1,
      },
      end: {
        x: x2,
        y: y2,
      },
    };
  }

  updateTool () {
    if (!this.initial) return
    if (this.hidden) {
      if (this.layer.annotationStore.selectedElementName === this.annotation.name) {
        this.layer.selectedElementName = null
      }
      this.outerLineContainer.remove()
    } else {
      if (this.layer.annotationStore.selectedElementName !== this.annotation.name) {
        this.layer.annotationStore.selectedElementName = this.annotation.name
      }
      this.container.append(this.outerLineContainer)
      this.outerLine.addEventListener('mousedown', this.onMousedown)
      this.outerLine.addEventListener('touchstart', this.onMousedown)

      this.eventBus.dispatch('propertyChange', this.annotation)
    }
  }

  handleClick (event) {
    console.log(this.annotation)
    if (!this.hidden) return
    if (this.layer.toolMode === 'form') {
      this.hidden = false
      this.updateTool()
      
      if (!this.hidden && document.getElementsByClassName('right-panel')[0].className.indexOf('closed') !== -1) {
        document.getElementById('propertyPanelButton').click()
      }
      onClickOutside([this.annotationContainer, this.outerLine, this.deletetButton, document.getElementById('propertyPanelButton'), document.getElementsByClassName('right-panel')[0]], this.handleOutside.bind(this))
    } else {
      if (event.target && event.target.className === 'option') {
        const children = event.target.parentNode.children
        for (let i = 0; i < children.length; i++) {
          if (children[i] === event.target) {
            this.annotation.select = i + 1
            break
          }
        }
        this.handleSelectItemBg()
      }
    }
  }

  handleOutside () {
    if (!this.hidden && this.layer.annotationStore.selectedElementName === this.annotation.name) {
      document.getElementById('propertyPanelButton').click()
    }

    this.hidden = !this.hidden

    if (this.layer.annotationStore.selectedElementName === this.annotation.name) {
      this.layer.annotationStore.selectedElementName = null
    }
    this.outerLine.removeEventListener('mousedown', this.onMousedown)
    this.outerLine.removeEventListener('touchstart', this.onMousedown)
    this.outerLineContainer.remove()
  }

  handleMouseDown (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    const operatorId = event.target.getAttribute('data-id')
  
    const { pageX, pageY } = getClickPoint(event)
    this.startState = {
      operator: operatorId,
      clickX: pageX,
      clickY: pageY,
    }

    document.addEventListener('mousemove', this.onMousemove)
    document.addEventListener('mouseup', this.onMouseup)
    document.addEventListener('touchmove', this.onMousemove)
    document.addEventListener('touchend', this.onMouseup)

    this.handleClick()
  };

  handleMouseMove (event) {
    if (event.type === 'touchmove') {
      document.body.style.overscrollBehavior = 'none';
    }
    this.moving = true
    const { pageX, pageY } = getClickPoint(event)
    const start = this.start
    const end = this.end
    const startState = this.startState

    const { width, height } = this.viewport
    if (startState.operator === 'nw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.end.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(start.y - 5, y1)
      y1 = Math.max(0, y1)

      this.update({
        start: { x: x1, y: start.y },
        end: { x: end.x, y: y1 },
      });
    } else if (startState.operator === 'w-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)

      this.update({
        start: { x: x1, y: start.y },
        end,
      });
    } else if (startState.operator === 'sw-resize') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      x1 = Math.min(end.x - 5, x1)
      x1 = Math.max(0, x1)
      y1 = Math.min(height, y1)
      y1 = Math.max(end.y + 5, y1)

      this.update({
        start: { x: x1, y: y1 },
        end,
      });
    } else if (startState.operator === 's-resize') {
      let y1 = pageY - (startState.clickY - this.start.y);
      y1 = Math.min(height, y1)
      y1 = Math.max(end.y + 5, y1)
      this.update({
        start: { x: start.x, y: y1 },
        end,
      });
    } else if (startState.operator === 'se-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.start.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(height, y)
      y = Math.max(end.y + 5, y)
      this.update({
        start: { x: start.x, y },
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'e-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      this.update({
        start,
        end: { x, y: end.y },
      });
    } else if (startState.operator === 'ne-resize') {
      let x = pageX - (startState.clickX - this.end.x);
      let y = pageY - (startState.clickY - this.end.y);
      x = Math.min(width, x)
      x = Math.max(start.x + 5, x)
      y = Math.min(start.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start,
        end: { x, y },
      });
    } else if (startState.operator === 'n-resize') {
      let y = pageY - (startState.clickY - this.end.y);
      y = Math.min(start.y - 5, y)
      y = Math.max(0, y)
      this.update({
        start,
        end: { x: end.x, y },
      });
    } else if (startState.operator === 'move') {
      let x1 = pageX - (startState.clickX - this.start.x);
      let y1 = pageY - (startState.clickY - this.start.y);
      let x2 = pageX - (startState.clickX - this.end.x);
      let y2 = pageY - (startState.clickY - this.end.y);

      const rect = {
        width: Math.abs(start.x -end.x),
        height: Math.abs(start.y -end.y)
      }
      if (x1 < x2) {
        x1 = Math.max(0, x1)
        x1 = Math.min(width - rect.width, x1)
        x2 = Math.max(rect.width, x2)
        x2 = Math.min(width, x2)
      } else {
        x2 = Math.max(0, x2)
        x2 = Math.min(width - rect.width, x2)
        x1 = Math.max(rect.width, x1)
        x1 = Math.min(width, x1)
      }

      if (y1 < y2) {
        y1 = Math.max(0, y1)
        y1 = Math.min(height - rect.height, y1)
        y2 = Math.max(rect.height, y2)
        y2 = Math.min(height, y2)
      } else {
        y2 = Math.max(0, y2)
        y2 = Math.min(height - rect.height, y2)
        y1 = Math.max(rect.height, y1)
        y1 = Math.min(height, y1)
      }

      this.update({
        start: { x: x1, y: y1 },
        end: { x: x2, y: y2 },
      });
    }
  }

  handleDelete (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    this.handleOutside()
    this.annotationContainer.remove()

    this.annotation.isDelete = true
    this.eventBus.dispatch('annotationChange', {
      type: 'delete',
      annotation: {
        operate: "del-annot",
        name: this.annotation.name,
        page: this.page
      }
    })
  }

  handleMouseUp (event) {
    if (this.layer.tool) {
      event.stopPropagation()
    }
    if (event.type === 'touchend') {
      document.body.style.overscrollBehavior = 'auto';
    }
    this.moving = false
    document.removeEventListener('mousemove', this.onMousemove)
    document.removeEventListener('mouseup', this.onMouseup)
    document.removeEventListener('touchmove', this.onMousemove)
    document.removeEventListener('touchend', this.onMouseup)
    
    const { pageX, pageY } = getClickPoint(event)
    if (pageX === this.startState.clickX && pageY === this.startState.clickY) return

    this.start = this.newStart
    this.end = this.newEnd

    const annotation = this.annotation
    const { start, end } = this.getInitialPoint()

    const linePoints = start.x + ',' + start.y + ',' + end.x + ',' + end.y
    if (linePoints === annotation.linePoints) return
    annotation.linePoints = linePoints

    this.eventBus.dispatch('annotationChange', {
      type: 'modify',
      annotation: {
        operate: "mod-form",
        name: annotation.name,
        page: this.page,
        rect: linePoints
      }
    })
  }

  update ({ start, end }) {
    const rect = this.calculate(start, end)

    this.newStart = start
    this.newEnd = end

    const actualbdwidth = (this.annotation.lineWidth || this.annotation.width || 2) * this.scale

    this.annotationContainer.style.top = rect.top + 'px'
    this.annotationContainer.style.left = rect.left + 'px'
    this.annotationContainer.style.width = rect.width + 'px'
    this.annotationContainer.style.height = rect.height + 'px'
    
    this.annotationContainer.style.width = `${rect.width}px`;
    this.annotationContainer.style.height = `${rect.height}px`;

    this.borderElement.style.width = `${rect.width}px`;
    this.borderElement.style.height = `${rect.height}px`;

    this.selectElement.style.width = `${rect.width}px`;
    this.selectElement.style.height = `${rect.height}px`;

    this.outerLine.style.left = `${rect.left - 8}px`
    this.outerLine.style.top = `${rect.top - 8}px`
    this.outerLine.style.width = `${rect.width + 8 * 2}px`
    this.outerLine.style.height = `${rect.height + 8 * 2}px`

    const left = (rect.left + rect.width - 30) + 'px'
    const top = (rect.top + rect.height + 8) + 'px'
    this.deletetButton.style.left = left
    this.deletetButton.style.top = top
    
    this.moveRect.setAttribute('width',  rect.width + 9)
    this.moveRect.setAttribute('height',  rect.height + 9)
    
    this.LeftRect.setAttribute("y", rect.height / 2 + 5)

    this.bottomLeftRect.setAttribute("y", rect.height + 9)

    this.bottomRect.setAttribute("x", rect.width / 2 + 5)
    this.bottomRect.setAttribute("y", rect.height + 9)


    this.bottomRightRect.setAttribute("x", rect.width + 9)
    this.bottomRightRect.setAttribute("y", rect.height + 9)


    this.rightRect.setAttribute("x", rect.width + 9)
    this.rightRect.setAttribute("y", rect.height / 2 + 5)

    
    this.topRightRect.setAttribute("x", rect.width + 9)

    this.topRect.setAttribute("x", rect.width / 2 + 5)
  }

  handlePropertyPanel (props) {
    if (this.layer.annotationStore.selectedElementName !== this.annotation.name) return

    let updatePropsNum = 0
    for (const key in props) {
      for (const item in this.annotation) {
        if (item === key && key === 'options' && JSON.stringify(this.annotation[item]) !== JSON.stringify(props[key])) {
          props[key] = Array.from(props[key])
          this.annotation[item] = props[key]
          updatePropsNum ++

          this.selectElement.innerText = ''
          for (var i = 0; i < this.annotation.options.length; i++) {
            const optionElement = createElement(
              'div',
              {
                width: '100%',
                height: '30px',
                lineHeight: '30px',
                borderBottom: '1px solid #666666',
                overflow: 'hidden',
                textOverflow: 'ellipsis',
                whiteSpace: 'nowrap'
              },
              {
                class: 'option'
              }
            )
            optionElement.innerText = this.annotation.options[i]
            if (this.annotation.select - 1 === i) {
              optionElement.style.backgroundColor = '#dcdcdc'
            }
            this.selectElement.appendChild(optionElement)
          }
        }

        if (item === key && this.annotation[item] !== props[key] && key !== 'options') {
          this.annotation[item] = props[key]
          updatePropsNum ++

          if (key === 'backgroundColor') {
            this.borderElement.style[key] = props[key]
            if (props[key] === '') {
              props.clearBackgroundColor = true
            }
          }
          if (key === 'fontSize') {
            this.selectElement.style.fontSize = props[key] + 'px'
          }
          if (key === 'textColor') {
            this.selectElement.style.color = props[key]
          }
          if (key === 'fontName') {
            this.selectElement.style.fontFamily = props[key]
          }
        }
      }
    }

    if (updatePropsNum > 0) {
      const annotation = {
        operate: "mod-form",
        name: this.annotation.name,
        page: this.page
      }
      if (props.options) {
        props.options = props.options.join(',')
        if (props.options === '') {
          props.clearOption = true
        }
      }
      if (props.select && typeof props.select === 'number') {
        props.select += ''
      }
      const updateAnnot = Object.assign({}, annotation, props)
      this.eventBus.dispatch('annotationChange', {
        type: 'modify',
        annotation: updateAnnot
      })
    }
  }

  setProperty (props) {
    if (props.name !== this.annotation.name) return

    for (const key in props) {
      for (const item in this.annotation) {
        if (item === key && this.annotation[item] !== props[key]) {
          this.annotation[item] = props[key]

          if (key === 'backgroundColor') {
            this.borderElement.style[key] = props[key]
          }
        }
      }
    }

    const annotation = {
      operate: "mod-form",
      page: this.page
    }
    const updateAnnot = Object.assign({}, annotation, props)
    this.eventBus.dispatch('annotationChange', {
      type: 'modify',
      annotation: updateAnnot
    })
  }

  changeSelectAvailable () {
    if (this.layer.toolMode === 'form') {
      this.selectElement.style.pointerEvents = 'none'
    } else {
      this.selectElement.style.pointerEvents = 'auto'
    }
  }

  handleSelectItemBg () {
    for (const opt of this.selectElement.children) {
      opt.style.backgroundColor = 'unset'
    }
    if (this.selectElement.children[this.annotation.select - 1]) {
      this.selectElement.children[this.annotation.select - 1].style.backgroundColor = '#dcdcdc'
    }
    this.setProperty({
      select: this.annotation.select + '',
      name: this.annotation.name
    })
  }
}
